@@ -17,6 +17,7 @@ import calendar
 step_search_re = re.compile(r'^([^-]+)-([^-/]+)(/(\d+))?$')
 only_int_re = re.compile(r'^\d+$')
 star_or_int_re = re.compile(r'^(\d+|\*)$')
+special_weekday_re = re.compile(r'^(\w+)#(\d+)|l(\d+)$')
 VALID_LEN_EXPRESSION = [5, 6]
 
 
@@ -42,6 +43,14 @@ class CroniterBadCronError(CroniterError):
     pass
 
 
+class CroniterUnsupportedSyntaxError(CroniterBadCronError):
+    """ Valid cron syntax, but likely to produce inaccurate results """
+    # Extending CroniterBadCronError, which may be contridatory, but this allows
+    # catching both errors with a single exception.  From a user perspective
+    # these will likely be handled the same way.
+    pass
+
+
 class CroniterBadDateError(CroniterError):
     """ Unable to find next/prev timestamp match """
     pass
@@ -385,14 +394,14 @@ class croniter(object):
 
             candidates = []
             for wday, nth in nth_weekday_of_month.items():
-                w = (wday + 6) % 7
-                c = calendar.Calendar(w).monthdayscalendar(d.year, d.month)
-                if c[0][0] == 0:
-                    c.pop(0)
+                c = self._get_nth_weekday_of_month(d.year, d.month, wday)
                 for n in nth:
-                    if len(c) < n:
+                    if n == "l":
+                        candidate = c[-1]
+                    elif len(c) < n:
                         continue
-                    candidate = c[n - 1][0]
+                    else:
+                        candidate = c[n - 1]
                     if (
                         (is_prev and candidate <= d.day) or
                         (not is_prev and d.day <= candidate)
@@ -530,9 +539,23 @@ class croniter(object):
             if c <= range_val:
                 candidate = c
                 break
-
+        if candidate > range_val:
+            # fix crontab "0 6 30 3 *" condidates only a element,
+            # then get_prev error return 2021-03-02 06:00:00
+            return - x
         return (candidate - x - range_val)
 
+    @staticmethod
+    def _get_nth_weekday_of_month(year, month, day_of_week):
+        """ For a given year/month return a list of days in nth-day-of-month order.
+        The last weekday of the month is always [-1].
+        """
+        w = (day_of_week + 6) % 7
+        c = calendar.Calendar(w).monthdayscalendar(year, month)
+        if c[0][0] == 0:
+            c.pop(0)
+        return tuple(i[0] for i in c)
+
     def is_leap(self, year):
         if year % 400 == 0 or (year % 4 == 0 and year % 100 != 0):
             return True
@@ -560,10 +583,25 @@ class croniter(object):
                 e = e_list.pop()
 
                 if i == 4:
-                    e, sep, nth = str(e).partition('#')
-                    if nth and not re.match(r'[1-5]', nth):
-                        raise CroniterBadCronError(
-                            "[{0}] is not acceptable".format(expr_format))
+                    # Handle special case in the day-of-week expression
+                    m = special_weekday_re.match(str(e))
+                    if m:
+                        orig_e = e
+                        e, nth, last = m.groups()
+                        if nth:
+                            try:
+                                nth = int(nth)
+                                assert (nth >= 1 and nth <= 5)
+                            except (ValueError, AssertionError):
+                                raise CroniterBadCronError(
+                                    "[{0}] is not acceptable.  Invalid day_of_week "
+                                    "value: '{1}'".format(expr_format, orig_e))
+                        elif last:
+                            nth = "l"
+                            e = last
+                        del last, orig_e
+                    else:
+                        nth = None
 
                 # Before matching step_search_re, normalize "*" to "{min}-{max}".
                 # Example: in the minute field, "*/5" normalizes to "0-59/5"
@@ -618,7 +656,7 @@ class croniter(object):
                         raise CroniterBadCronError(
                             'invalid range: {0}'.format(exc))
                     e_list += (["{0}#{1}".format(item, nth) for item in rng]
-                               if i == 4 and nth else rng)
+                               if i == 4 and nth and nth != "l" else rng)
                 else:
                     if t.startswith('-'):
                         raise CroniterBadCronError((
@@ -650,7 +688,7 @@ class croniter(object):
                     if i == 4 and nth:
                         if t not in nth_weekday_of_month:
                             nth_weekday_of_month[t] = set()
-                        nth_weekday_of_month[t].add(int(nth))
+                        nth_weekday_of_month[t].add(nth)
 
             res = set(res)
             res = sorted(res, key=lambda i: "{:02}".format(i) if isinstance(i, int) else i)
@@ -661,6 +699,16 @@ class croniter(object):
                                       and res[0] == '*')
                             else res)
 
+        # Check to make sure the dow combo in use is supported
+        if nth_weekday_of_month:
+            dow_expanded_set = set(expanded[4])
+            dow_expanded_set = dow_expanded_set.difference(nth_weekday_of_month.keys())
+            dow_expanded_set.discard("*")
+            if dow_expanded_set:
+                raise CroniterUnsupportedSyntaxError(
+                    "day-of-week field does not support mixing literal values and nth day of week syntax.  "
+                    "Cron: '{}'    dow={} vs nth={}".format(expr_format, dow_expanded_set, nth_weekday_of_month))
+
         return expanded, nth_weekday_of_month
 
     @classmethod
@@ -698,7 +746,8 @@ class croniter(object):
         return (max(tdp, tdt) - min(tdp, tdt)).total_seconds() < 60
 
 
-def croniter_range(start, stop, expr_format, ret_type=None, day_or=True, exclude_ends=False):
+def croniter_range(start, stop, expr_format, ret_type=None, day_or=True, exclude_ends=False,
+                   _croniter=None):
     """
     Generator that provides all times from start to stop matching the given cron expression.
     If the cron expression matches either 'start' and/or 'stop', those times will be returned as
@@ -707,6 +756,7 @@ def croniter_range(start, stop, expr_format, ret_type=None, day_or=True, exclude
     You can think of this function as sibling to the builtin range function for datetime objects.
     Like range(start,stop,step), except that here 'step' is a cron expression.
     """
+    _croniter = _croniter or croniter
     auto_rt = datetime.datetime
     if type(start) != type(stop):
         raise TypeError("The start and stop must be same type.  {0} != {1}".
@@ -725,8 +775,8 @@ def croniter_range(start, stop, expr_format, ret_type=None, day_or=True, exclude
             start += ms1
             stop -= ms1
     year_span = math.floor(abs(stop.year - start.year)) + 1
-    ic = croniter(expr_format, start, ret_type=datetime.datetime, day_or=day_or,
-                  max_years_between_matches=year_span)
+    ic = _croniter(expr_format, start, ret_type=datetime.datetime, day_or=day_or,
+                   max_years_between_matches=year_span)
     # define a continue (cont) condition function and step function for the main while loop
     if start < stop:        # Forward
         def cont(v):
